package rozdzial14c;

import javafx.animation.AnimationTimer;
import javafx.application.Application;
import javafx.scene.Scene;
import javafx.scene.layout.StackPane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import javafx.stage.Stage;
import java.time.Duration;
import java.time.Instant;
import java.util.concurrent.TimeUnit;
/**
 * @author Jacek Piechota
 <p>sprawdzamy rozwiązanie z klasy {@code Listing14c_04c}.</p>
 <p>Przedłużamy 2&times; czas trwania animacji.</p>
 <p>Pomiary wykonujemy wykorzystując {@code System.nanotime()}.</p>
 <p>Po uruchomieniu klasy na konsoli zobaczymy na przykład:</p>
 <pre>
 Czas trwania: 4011
 Counter: 116
 </pre>
 <p>Jak widzimy czas trwania animacji wzrósł dwukrotnie podobnie jak liczba klatek animacji.</p>
 */
public class Listing14c_04d extends Application {
    private double radius;
    private Circle circle;
    private long duration;
    private Linear interp;

    @Override
    public void start(Stage stage) {
        StackPane root = new StackPane();
        circle = new Circle(radius);
        duration = 4000;//ms
        circle.setFill(Color.DARKCYAN);
        interp = new Linear();
        root.getChildren().add(circle);
        AnimationTimer timer = new MyTimer();
        timer.start();
        Scene scene = new Scene(root, 300, 300);
        stage.setTitle("AnimationTimer");
        stage.setScene(scene);
        stage.show();
    }

    private class MyTimer extends AnimationTimer {
        Instant start = Instant.now();
        long start2 = System.nanoTime();
        int counter = 0;
        @Override
        public void handle(long now) {
            long elapsed1 = now - start2;
            long elapsed2 = TimeUnit.MILLISECONDS.convert(elapsed1,
                    TimeUnit.NANOSECONDS);
            counter++;
            if(elapsed2 <= duration) {
                double radiusStart = 10.0;
                double radiusEnd = 80;
                double r = interp.interpolate(radiusStart, radiusEnd,
                        (double) elapsed2 / (double) duration);
                circle.radiusProperty().set(r);
                try {
                    Thread.sleep(30);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }else{
                radius = 10.0;
                circle.radiusProperty().set(radius);
                stop();
                Instant end = Instant.now();
                long millis = Duration.between(start,  end).toMillis();
                System.out.println("Czas trwania: "+ millis);
                System.out.println("Counter: "+ counter);
            }
        }
    }

    public static void main(String[] args) {
        Application.launch(args);
    }
}